The single error type shared across every sketch family in this crate.
Enums
- Sketch
Error - Error type shared across all sketch families (HLL, Theta, CPC) in this crate — there is no per-family error type.
The single error type shared across every sketch family in this crate.