pub trait ThetaInput: Sealed { }Expand description
Any of the three theta sketch types can be fed into a ThetaUnion,
ThetaIntersection, ThetaAnotB, or jaccard_similarity. This trait
is sealed — it cannot be implemented outside this crate — since the
set-op shims only have concrete overloads for these three exact types.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".