pub trait TupleInput<S: TupleSummary>: Sealed { }Expand description
Either generic Tuple sketch type can be fed into this module’s set operations. Sealed — the shims have concrete overloads for these two types only.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".