pub trait ArrayOfDoublesInput: Sealed {
// Required method
fn get_num_values(&self) -> u8;
}Expand description
Either ArrayOfDoubles sketch type can be fed into any of this module’s set operations — union, intersection, a-not-b, and Jaccard similarity. This trait is sealed — it cannot be implemented outside this crate — since the set-op shims only have concrete overloads for these two exact types.
Required Methods
Sourcefn get_num_values(&self) -> u8
fn get_num_values(&self) -> u8
The fixed number of f64 values each of this sketch’s retained
entries carries. Set operations require all operands to agree.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".